//Phosphate glass
#include <iostream>
#include <fstream>
#include <cmath>
#include <cstdlib>
#include <string>
#include "NaCaPO.h"

using namespace std;

int main(int argc, char **argv)
{
    if(argc<4)
    {
	cerr << "Usage: " << argv[0] << " <nsteps> <output> <P-O cutoff (A)>" << endl;
	return 1;
    }

    const int nsteps = atoi(argv[1]);
    const int natoms =  nP+nO+nNa+nCa;
//    const int natomss = natoms+nO;
	
    double poss[natoms][3];//, pos[natoms][3];
    int nsps[natoms];

    int atom1, atom2;
    double dx, dy, dz, dr;

    const int nsp = 4;
    int n[nsp];
    n[0] = nP;
    n[1] = nO;
	n[2] = nNa;
	n[3] = nCa;;
	
    double c[nsp];
    for (int i=0; i<nsp; i++)
    {
        c[i] = double(n[i])/double(natoms);
    }

////////////////

    int offset[nsp];
    offset[0] = 0;
    for (int i=1; i<nsp; i++)
    {
      offset[i] = n[i-1] + offset[i-1];
    }

    const double cutoff = atof(argv[3]);

    int OPcoord[natoms];
    int OPc;
    for (int i=0; i<natoms; i++)
    {
	OPcoord[i] = 0;
    }

    const int maxqn = 10;

    int Pqn[maxqn];
    int qn;
    for (int i=0; i<maxqn; i++)
    {
	Pqn[i] = 0;
    }

    int ti, tii;
	ifstream fin;
    fin.open(argv[2]);

    // Loop round steps
    char str[101];
	

    for (int i=0; i<nsteps; i++)
    {
      if(i%100==0){cerr << i << " ";}
      
      for (int j=0; j<9; j++){
        fin.getline(str,101);
      }

      // Read in positions

      for (int j=0; j<natoms; j++)
      {
        fin >> ti >> nsps[j] >> poss[j][0] >> poss[j][1] >> poss[j][2];
        fin.getline(str,101);
      }

      // construct O-P coordination numbers
    
	for (int j=0; j<natoms; j++){

	    OPc = 0;

	    for (int k=0; k<natoms; k++){
 
		atom1 = j;
		atom2 = k;
	
                if(atom1!=atom2 && nsps[atom1]==2 && nsps[atom2]==1){ // Ocore - P
	
                  dx = poss[atom1][0] - poss[atom2][0];
                  dy = poss[atom1][1] - poss[atom2][1];
                  dz = poss[atom1][2] - poss[atom2][2];

                  while(dx>0.5){dx-=1.0;}
                  while(dy>0.5){dy-=1.0;}
                  while(dz>0.5){dz-=1.0;}
                  while(dx<-0.5){dx+=1.0;}
                  while(dy<-0.5){dy+=1.0;}
                  while(dz<-0.5){dz+=1.0;}

		  dr = cell*sqrt(dx*dx + dy*dy + dz*dz);
		  
                  if(dr<cutoff){OPc++;}
               }  
	    }

	    OPcoord[j] = OPc;
	}

	// are the P-O oxygens connected to P?
	
	for (int j=0; j<natoms; j++)
	{
	    qn = 0;

	    for (int k=0; k<natoms; k++)
	    {
		atom1 = j;
		atom2 = k; 

                if(atom1!=atom2 && nsps[atom1]==1 && nsps[atom2]==2){ // P - OCore
		
                  dx = poss[atom1][0] - poss[atom2][0];
                  dy = poss[atom1][1] - poss[atom2][1];
                  dz = poss[atom1][2] - poss[atom2][2];

                  while(dx>0.5){dx-=1.0;}
                  while(dy>0.5){dy-=1.0;}
                  while(dz>0.5){dz-=1.0;}
                  while(dx<-0.5){dx+=1.0;}
                  while(dy<-0.5){dy+=1.0;}
                  while(dz<-0.5){dz+=1.0;}

		  dr = cell*sqrt(dx*dx + dy*dy + dz*dz);

		  if(dr<cutoff)
		  {
		      // then the P-O are bonded

		      // is the O bridging?

		      if(OPcoord[k]>=2){qn++;}
		  }
               }
	    }
	    if(nsps[atom1]==1){Pqn[qn]++;}
	}

    }

    fin.close();

    // output

    ofstream fout;

    fout.open("qnP.out");

    const double dnsteps = double(nsteps);
    const double dnPst = double(nP)*dnsteps;

    double NCP = 0.0;

    for (int i=0; i<maxqn; i++)
    {
	fout << i << " " << Pqn[i]/dnPst << endl;
        NCP += i*Pqn[i]/dnPst;
    }

    fout << NCP << endl;

    fout.close();

    return 0;
}
